@@ -250,6 +250,8 @@ static int dh_handshake(ssh_session session) {
       switch(session->next_crypto->kex_type){
         case SSH_KEX_DH_GROUP1_SHA1:
         case SSH_KEX_DH_GROUP14_SHA1:
+        case SSH_KEX_DH_GROUP16_SHA512:
+        case SSH_KEX_DH_GROUP18_SHA512:
           rc = ssh_client_dh_init(session);
           break;
 #ifdef HAVE_ECDH
@@ -295,7 +297,7 @@ static int dh_handshake(ssh_session session) {
 static int ssh_service_request_termination(void *s){
   ssh_session session = (ssh_session)s;
   if(session->session_state == SSH_SESSION_STATE_ERROR ||
-      session->auth_service_state != SSH_AUTH_SERVICE_SENT)
+      session->auth.service_state != SSH_AUTH_SERVICE_SENT)
     return 1;
   else
     return 0;
@@ -319,7 +321,7 @@ static int ssh_service_request_termination(void *s){
 int ssh_service_request(ssh_session session, const char *service) {
   int rc=SSH_ERROR;
 
-  if(session->auth_service_state != SSH_AUTH_SERVICE_NONE)
+  if(session->auth.service_state != SSH_AUTH_SERVICE_NONE)
     goto pending;
 
   rc = ssh_buffer_pack(session->out_buffer,
@@ -330,7 +332,7 @@ int ssh_service_request(ssh_session session, const char *service) {
       ssh_set_error_oom(session);
       return SSH_ERROR;
   }
-  session->auth_service_state=SSH_AUTH_SERVICE_SENT;
+  session->auth.service_state = SSH_AUTH_SERVICE_SENT;
   if (ssh_packet_send(session) == SSH_ERROR) {
     ssh_set_error(session, SSH_FATAL,
         "Sending SSH2_MSG_SERVICE_REQUEST failed.");
@@ -345,7 +347,7 @@ pending:
   if (rc == SSH_ERROR) {
       return SSH_ERROR;
   }
-  switch(session->auth_service_state){
+  switch(session->auth.service_state) {
   case SSH_AUTH_SERVICE_DENIED:
     ssh_set_error(session,SSH_FATAL,"ssh_auth_service request denied");
     break;
@@ -409,6 +411,14 @@ static void ssh_client_connection_callback(ssh_session session)
 
             ssh_packet_set_default_callbacks(session);
             session->session_state = SSH_SESSION_STATE_INITIAL_KEX;
+            rc = ssh_set_client_kex(session);
+            if (rc != SSH_OK) {
+                goto error;
+            }
+            rc = ssh_send_kex(session, 0);
+            if (rc < 0) {
+                goto error;
+            }
             set_status(session, 0.5f);
 
             break;
@@ -418,14 +428,19 @@ static void ssh_client_connection_callback(ssh_session session)
         case SSH_SESSION_STATE_KEXINIT_RECEIVED:
             set_status(session,0.6f);
             ssh_list_kex(&session->next_crypto->server_kex);
-            if (ssh_set_client_kex(session) < 0) {
-                goto error;
+            if (session->next_crypto->client_kex.methods[0] == NULL) {
+                /* in rekeying state if next_crypto client_kex is empty */
+                rc = ssh_set_client_kex(session);
+                if (rc != SSH_OK) {
+                    goto error;
+                }
+                rc = ssh_send_kex(session, 0);
+                if (rc < 0) {
+                    goto error;
+                }
             }
             if (ssh_kex_select_methods(session) == SSH_ERROR)
                 goto error;
-            if (ssh_send_kex(session, 0) < 0) {
-                goto error;
-            }
             set_status(session,0.8f);
             session->session_state=SSH_SESSION_STATE_DH;
             if (dh_handshake(session) == SSH_ERROR) {
@@ -700,7 +715,7 @@ error:
   if (session->out_hashbuf) {
     ssh_buffer_reinit(session->out_hashbuf);
   }
-  session->auth_methods = 0;
+  session->auth.supported_methods = 0;
   SAFE_FREE(session->serverbanner);
   SAFE_FREE(session->clientbanner);
 
